@@ -18,10 +18,13 @@
 #include "util.h"
 #include "value.h"
 #include "vector.h"
+#include "search.h"
+#include "probe.h"
 
 // macros
 
 #define THROUGH(piece) ((piece)==Empty)
+#define ABS(x) ((x)<0?-(x):(x))
 
 // constants and variables
 
@@ -46,7 +49,7 @@ const int KnightOutpostMatrix[2][64] = {
 };
 
 static int lazy_eval_cutoff = 200; /* Thomas */
-static bool LazyEval = false; // true
+static bool LazyEval = true; // true
 static bool KingSafety = false; // true
 static int KingSafetyMargin = 1600;
 static bool king_is_safe[ColourNb];
@@ -70,10 +73,8 @@ static const int BishopMobOpening = 5;
 static const int BishopMobEndgame = 5;
 static const int RookMobOpening = 2;
 static const int RookMobEndgame = 4;
-static const int QueenMobOpening = 1;
+static const int QueenMobOpening = 1; //was 1
 static const int QueenMobEndgame = 2;
-static const int KingMobOpening = 0;
-static const int KingMobEndgame = 0;
 
 static const bool UseOpenFile = true;
 static const int RookSemiOpenFileOpening = 10;
@@ -85,11 +86,10 @@ static const int RookKingFileOpening = 20;
 
 static const bool UseKingAttack = true;
 static const int KingAttackOpening = 20; // was 20
-static const int KingAttackOpening1 = 20; // was 20
 
 static const bool UseShelter = true;
 static const int ShelterOpening = 256; // 100%
-static const bool UseStorm = true;
+static const bool UseStorm = true; // was true
 static const int StormOpening = 10;
 
 static const int Rook7thOpening = 20;
@@ -99,11 +99,16 @@ static const int Queen7thEndgame = 20;
 
 static const int TrappedBishop = 100;
 
+static const int PawnBlockedBishop = 10; 
+
+static const int EndgamePawnKnight = 10;
+static const int EndgamePassedPawnKnight = 10;
+
 static const int BlockedBishop = 50;
 static const int BlockedRook = 50;
 
 static const int ProtectedPassedOpening = 10;
-static const int ProtectedPassedEndgame = 20;
+static const int ProtectedPassedEndgame = 10;
 
 static const int PassedOpeningMin = 10;
 static const int PassedOpeningMax = 70;
@@ -122,6 +127,9 @@ static const int KingAttackWeight[16] = {
    0, 0, 128, 192, 224, 240, 248, 252, 254, 255, 256, 256 ,256, 256, 256, 256,
 };
 
+static const int SSpaceWeight[16] = {
+   0, 0, 15, 30, 60, 90, 90, 90, 90, 90, 90, 90 ,90, 90, 90, 90,
+};
 
 // variables
 
@@ -202,12 +210,12 @@ void eval_init() {
 
    MobUnit[White][Empty] = MobMove;
 
-   MobUnit[White][BP] = MobAttack;
-   MobUnit[White][BN] = MobAttack;
-   MobUnit[White][BB] = MobAttack;
-   MobUnit[White][BR] = MobAttack;
-   MobUnit[White][BQ] = MobAttack;
-   MobUnit[White][BK] = MobAttack;
+   MobUnit[White][BP] = 1; //MobAttack;
+   MobUnit[White][BN] = 1; //MobAttack;
+   MobUnit[White][BB] = 1; //MobAttack;
+   MobUnit[White][BR] = 1; //MobAttack;
+   MobUnit[White][BQ] = 1; //MobAttack;
+   MobUnit[White][BK] = 1; //MobAttack;
 
    MobUnit[White][WP] = MobDefense;
    MobUnit[White][WN] = MobDefense;
@@ -218,12 +226,12 @@ void eval_init() {
 
    MobUnit[Black][Empty] = MobMove;
 
-   MobUnit[Black][WP] = MobAttack;
-   MobUnit[Black][WN] = MobAttack;
-   MobUnit[Black][WB] = MobAttack;
-   MobUnit[Black][WR] = MobAttack;
-   MobUnit[Black][WQ] = MobAttack;
-   MobUnit[Black][WK] = MobAttack;
+   MobUnit[Black][WP] = 1; //MobAttack;
+   MobUnit[Black][WN] = 1; //MobAttack;
+   MobUnit[Black][WB] = 1; //MobAttack;
+   MobUnit[Black][WR] = 1; //MobAttack;
+   MobUnit[Black][WQ] = 1; //MobAttack;
+   MobUnit[Black][WK] = 1; //MobAttack;
 
    MobUnit[Black][BP] = MobDefense;
    MobUnit[Black][BN] = MobDefense;
@@ -238,12 +246,14 @@ void eval_init() {
       KingAttackUnit[piece] = 0;
    }
 
+	KingAttackUnit[WP] = 1;
    KingAttackUnit[WN] = 1;
    KingAttackUnit[WB] = 1;
    KingAttackUnit[WR] = 2;
    KingAttackUnit[WQ] = 4;
 
-   KingAttackUnit[BN] = 1;
+   KingAttackUnit[BP] = 1;
+	KingAttackUnit[BN] = 1;
    KingAttackUnit[BB] = 1;
    KingAttackUnit[BR] = 2;
    KingAttackUnit[BQ] = 4;
@@ -251,7 +261,7 @@ void eval_init() {
 
 // eval()
 
-int eval(const board_t * board, int alpha, int beta) {
+int eval(board_t * board, int alpha, int beta, bool in_pv) {
 
    int opening, endgame;
    material_info_t mat_info[1];
@@ -260,34 +270,25 @@ int eval(const board_t * board, int alpha, int beta) {
    int phase;
    int eval;
    int wb, bb;
-   int lazy_eval // Thomas
-
+   int lazy_eval, probe_score; // Thomas
+	
    ASSERT(board!=NULL);
 
    ASSERT(board_is_legal(board));
    ASSERT(!board_is_check(board)); // exceptions are extremely rare
 
-
-   /* lazy Cutoff Thomas */
-/*   if (LazyEval && board->piece_size[White] > 3 && board->piece_size[Black] > 3){
-
-	 lazy_eval = board->piece_material[White] - board->piece_material[Black];
-
-     ASSERT(eval>=-ValueEvalInf&&eval<=+ValueEvalInf);
-
-	 if (COLOUR_IS_BLACK(board->turn)) lazy_eval = -lazy_eval;
-	 if (lazy_eval - lazy_eval_cutoff >= beta)
-		return (lazy_eval);
-	 if (lazy_eval + lazy_eval_cutoff <= alpha)
-		return (lazy_eval);  
-	 // ende lazy cuttoff 
-	}  */
-
    // init
 
-   opening = 0;
+	opening = 0;
    endgame = 0;
 
+/*	if (egbb_is_loaded && board->piece_nb <=5 && ABS(SearchBest[0].value) >= WIN_SCORE) {
+		if (probe_bitbases(board, probe_score)){
+			return probe_score;
+		}
+	} */
+
+
    // material
 
    material_get_info(mat_info,board);
@@ -312,12 +313,12 @@ int eval(const board_t * board, int alpha, int beta) {
 
    // draw
 
-   eval_draw(board,mat_info,pawn_info,mul);
+	eval_draw(board,mat_info,pawn_info,mul);
 
-   if (mat_info->mul[White] < mul[White]) mul[White] = mat_info->mul[White];
-   if (mat_info->mul[Black] < mul[Black]) mul[Black] = mat_info->mul[Black];
+	if (mat_info->mul[White] < mul[White]) mul[White] = mat_info->mul[White];
+	if (mat_info->mul[Black] < mul[Black]) mul[Black] = mat_info->mul[Black];
 
-   if (mul[White] == 0 && mul[Black] == 0) return ValueDraw;
+	if (mul[White] == 0 && mul[Black] == 0) return ValueDraw; 
 
    // eval
 
@@ -335,7 +336,7 @@ int eval(const board_t * board, int alpha, int beta) {
 	eval_pattern(board,&opening,&endgame);
 
 	/* lazy Cutoff Thomas */
-   if (LazyEval && board->piece_size[White] > 3 && board->piece_size[Black] > 3){
+   if (LazyEval && board->piece_size[White] > 3 && board->piece_size[Black] > 3 && !in_pv){
 
 	 phase = mat_info->phase;
      lazy_eval = ((opening * (256 - mat_info->phase)) + (endgame * mat_info->phase)) / 256;
@@ -349,7 +350,6 @@ int eval(const board_t * board, int alpha, int beta) {
 		return (lazy_eval);  
 	 // ende lazy cuttoff 
    } 
-
    
    // eval
    eval_piece(board,mat_info,pawn_info,&opening,&endgame);
@@ -364,27 +364,27 @@ int eval(const board_t * board, int alpha, int beta) {
 
    // drawish bishop endgames
 
-   if ((mat_info->flags & DrawBishopFlag) != 0) {
+	if ((mat_info->flags & DrawBishopFlag) != 0) {
 
-      wb = board->piece[White][1];
-      ASSERT(PIECE_IS_BISHOP(board->square[wb]));
+		wb = board->piece[White][1];
+		ASSERT(PIECE_IS_BISHOP(board->square[wb]));
 
-      bb = board->piece[Black][1];
-      ASSERT(PIECE_IS_BISHOP(board->square[bb]));
+		bb = board->piece[Black][1];
+		ASSERT(PIECE_IS_BISHOP(board->square[bb]));
 
-      if (SQUARE_COLOUR(wb) != SQUARE_COLOUR(bb)) {
-         if (mul[White] == 16) mul[White] = 8; // 1/2
-         if (mul[Black] == 16) mul[Black] = 8; // 1/2
-      }
-   }
+		if (SQUARE_COLOUR(wb) != SQUARE_COLOUR(bb)) {
+			if (mul[White] == 16) mul[White] = 8; // 1/2
+			if (mul[Black] == 16) mul[Black] = 8; // 1/2
+		}
+	}
 
-   // draw bound
+	// draw bound
 
-   if (eval > ValueDraw) {
-      eval = (eval * mul[White]) / 16;
-   } else if (eval < ValueDraw) {
-      eval = (eval * mul[Black]) / 16;
-   }
+	if (eval > ValueDraw) {
+		eval = (eval * mul[White]) / 16;
+	} else if (eval < ValueDraw) {
+		eval = (eval * mul[Black]) / 16;
+	}
 
    // value range
 
@@ -644,19 +644,17 @@ static void eval_draw(const board_t * board, const material_info_t * mat_info, c
 
 static void eval_piece(const board_t * board, const material_info_t * mat_info, const pawn_info_t * pawn_info, int * opening, int * endgame) {
 
-   int colour;
    int op[ColourNb], eg[ColourNb];
    int me, opp;
-   int opp_flag;
    const sq_t * ptr;
    int from, to;
    int piece;
-   int mob;
+   int mob, penalty;
    int capture;
    const int * unit;
    int rook_file, king_file;
    int king;
-   int delta;
+   int delta, piece_nb;
    int king_rank,piece_rank,new_mob, piece_file;
 
    ASSERT(board!=NULL);
@@ -665,220 +663,496 @@ static void eval_piece(const board_t * board, const material_info_t * mat_info,
    ASSERT(opening!=NULL);
    ASSERT(endgame!=NULL);
 
-   // init
+   // eval White
 
-   for (colour = 0; colour < ColourNb; colour++) {
-      op[colour] = 0;
-      eg[colour] = 0;
+   me = White;
+	op[me] = 0;
+   eg[me] = 0;
+   opp = COLOUR_OPP(me);
+
+   unit = MobUnit[me];
+	piece_nb = 0;
+
+   // piece loop
+
+   for (ptr = &board->piece[me][1]; (from=*ptr) != SquareNone; ptr++) { // HACK: no king
+
+      piece = board->square[from];
+
+      switch (PIECE_TYPE(piece)) {
+
+      case Knight64:
+
+         // mobility
+
+         mob = -KnightUnit;
+
+         mob += unit[board->square[from-33]];
+         mob += unit[board->square[from-31]];
+         mob += unit[board->square[from-18]];
+         mob += unit[board->square[from-14]];
+         mob += unit[board->square[from+14]];
+         mob += unit[board->square[from+18]];
+         mob += unit[board->square[from+31]];
+         mob += unit[board->square[from+33]];
+
+         op[me] += mob * KnightMobOpening;
+         eg[me] += mob * KnightMobEndgame;
+
+			// outpost
+			mob = 0;
+			if (board->square[from-17] == WP)
+				mob += KnightOutpostMatrix[me][SquareTo64[from]]; 
+			if (board->square[from-15] == WP)
+				mob += KnightOutpostMatrix[me][SquareTo64[from]]; 
+		
+			penalty = 0;
+			if (((board->pawn_file[opp][FileA] | board->pawn_file[opp][FileB] | board->pawn_file[opp][FileC]) != 0)
+				&& ((board->pawn_file[opp][FileF] | board->pawn_file[opp][FileG] | board->pawn_file[opp][FileH]) != 0))
+				penalty -= EndgamePawnKnight;
+
+			if (((pawn_info->passed_bits[opp] & 224)) != 0 && ((pawn_info->passed_bits[opp] & 7) != 0))
+				penalty -= EndgamePassedPawnKnight;
+
+	
+			op[me] += mob;
+			//eg[me] += mob;
+			//op[me] += penalty / 2; 
+         eg[me] += penalty; 
+
+			if (PAWN_RANK(from,me) >= Rank5){
+				piece_nb++;
+			}
+			
+         break;
+
+      case Bishop64:
+
+         // mobility
+
+         mob = -BishopUnit;
+			penalty = 0;
+
+         for (to = from-17; capture=board->square[to], THROUGH(capture); to -= 17) mob += MobMove;
+         mob += unit[capture];
+			
+         for (to = from-15; capture=board->square[to], THROUGH(capture); to -= 15) mob += MobMove;
+         mob += unit[capture];
+			
+         for (to = from+15; capture=board->square[to], THROUGH(capture); to += 15) mob += MobMove;
+         mob += unit[capture];
+			if (capture == WP)
+				penalty += PawnBlockedBishop;
+
+         for (to = from+17; capture=board->square[to], THROUGH(capture); to += 17) mob += MobMove;
+         mob += unit[capture];
+			if (capture == WP)
+				penalty += PawnBlockedBishop;
+
+         op[me] += mob * BishopMobOpening - penalty;
+         eg[me] += mob * BishopMobEndgame - penalty;
+
+			mob = 0;
+			if (board->square[from-17] == WP)
+				mob += KnightOutpostMatrix[me][SquareTo64[from]]/2; 
+			if (board->square[from-15] == WP)
+				mob += KnightOutpostMatrix[me][SquareTo64[from]]/2; 
+			
+			op[me] += mob;
+			//eg[me] += mob;
+
+			if (PAWN_RANK(from,me) >= Rank5){
+				piece_nb++;
+			}
+
+         break;
+
+      case Rook64:
+
+         // mobility
+
+         mob = -RookUnit;
+
+         for (to = from-16; capture=board->square[to], THROUGH(capture); to -= 16) mob += MobMove;
+         mob += unit[capture];
+
+         for (to = from- 1; capture=board->square[to], THROUGH(capture); to -=  1) mob += MobMove;
+         mob += unit[capture];
+
+         for (to = from+ 1; capture=board->square[to], THROUGH(capture); to +=  1) mob += MobMove;
+         mob += unit[capture];
+
+         for (to = from+16; capture=board->square[to], THROUGH(capture); to += 16) mob += MobMove;
+         mob += unit[capture];
+
+         op[me] += mob * RookMobOpening;
+         eg[me] += mob * RookMobEndgame; 
+
+         // open file
+
+         if (UseOpenFile) {
+
+            op[me] -= RookOpenFileOpening / 2;
+            eg[me] -= RookOpenFileEndgame / 2;
+
+            rook_file = SQUARE_FILE(from);
+
+            if (board->pawn_file[me][rook_file] == 0) { // no friendly pawn
+
+               op[me] += RookSemiOpenFileOpening;
+               eg[me] += RookSemiOpenFileEndgame;
+
+               if (board->pawn_file[opp][rook_file] == 0) { // no enemy pawn
+                  op[me] += RookOpenFileOpening - RookSemiOpenFileOpening;
+                  eg[me] += RookOpenFileEndgame - RookSemiOpenFileEndgame;
+               }
+
+               if ((mat_info->cflags[opp] & MatKingFlag) != 0) {
+
+                  king = KING_POS(board,opp);
+                  king_file = SQUARE_FILE(king);
+
+                  delta = abs(rook_file-king_file); // file distance
+
+                  if (delta <= 1) {
+                     op[me] += RookSemiKingFileOpening;
+                     if (delta == 0) op[me] += RookKingFileOpening - RookSemiKingFileOpening;
+                  }
+               }
+            }
+         }
+
+         // 7th rank
+
+			if (PAWN_RANK(from,me) == Rank7) {
+					if ((pawn_info->flags[opp] & BackRankFlag) != 0 // opponent pawn on 7th rank
+					 || PAWN_RANK(KING_POS(board,opp),me) == Rank8) {
+						op[me] += Rook7thOpening;
+						eg[me] += Rook7thEndgame;
+					}
+			}
+
+			//if (PAWN_RANK(from,me) >= Rank7) op[me] += Rook7thOpening; 
+
+			if (PAWN_RANK(from,me) >= Rank5){
+				piece_nb++;
+			}
+
+			break;
+
+      case Queen64:
+
+         // mobility
+
+/*         mob = -QueenUnit;
+
+         for (to = from-17; capture=board->square[to], THROUGH(capture); to -= 17);// mob += MobMove;
+         mob += unit[capture];
+
+         for (to = from-16; capture=board->square[to], THROUGH(capture); to -= 16);// mob += MobMove;
+         mob += unit[capture];
+
+         for (to = from-15; capture=board->square[to], THROUGH(capture); to -= 15);// mob += MobMove;
+         mob += unit[capture];
+
+         for (to = from- 1; capture=board->square[to], THROUGH(capture); to -=  1);// mob += MobMove;
+         mob += unit[capture];
+
+         for (to = from+ 1; capture=board->square[to], THROUGH(capture); to +=  1);// mob += MobMove;
+         mob += unit[capture];
+
+         for (to = from+15; capture=board->square[to], THROUGH(capture); to += 15);// mob += MobMove;
+         mob += unit[capture];
+
+         for (to = from+16; capture=board->square[to], THROUGH(capture); to += 16);// mob += MobMove;
+         mob += unit[capture];
+
+         for (to = from+17; capture=board->square[to], THROUGH(capture); to += 17);// mob += MobMove;
+         mob += unit[capture];
+
+         op[me] += mob * QueenMobOpening;
+         eg[me] += mob * QueenMobEndgame;*/
+
+			king = KING_POS(board,opp);
+			king_file = SQUARE_FILE(king);
+			king_rank = SQUARE_RANK(king);
+			piece_file = SQUARE_FILE(from);
+			piece_rank = SQUARE_RANK(from); 
+						
+			new_mob = 10 - (abs(king_file-piece_file) + abs(king_rank-piece_rank)); 
+			
+			op[me] += new_mob;
+			eg[me] += new_mob; 
+
+         // 7th rank
+
+			if (PAWN_RANK(from,me) == Rank7) {
+					if ((pawn_info->flags[opp] & BackRankFlag) != 0 // opponent pawn on 7th rank
+					 || PAWN_RANK(KING_POS(board,opp),me) == Rank8) {
+						op[me] += Queen7thOpening;
+						eg[me] += Queen7thEndgame; 
+					}
+			}
+
+			//if (PAWN_RANK(from,me) >= Rank7) op[me] += Queen7thOpening; 
+
+			if (PAWN_RANK(from,me) >= Rank5){
+				piece_nb++;
+			}
+
+			break;
+
+      }
    }
 
-   // eval
+	op[me] += SSpaceWeight[piece_nb];
 
-   for (colour = 0; colour < ColourNb; colour++) {
+	// piece eval black
 
-      me = colour;
-      opp = COLOUR_OPP(me);
+   me = Black;
+	op[me] = 0;
+   eg[me] = 0;
+   opp = COLOUR_OPP(me);
 
-      opp_flag = COLOUR_FLAG(opp);
+   unit = MobUnit[me];
+	piece_nb = 0;
 
-      unit = MobUnit[me];
+   // piece loop
 
-      // piece loop
+   for (ptr = &board->piece[me][1]; (from=*ptr) != SquareNone; ptr++) { // HACK: no king
 
-      for (ptr = &board->piece[me][1]; (from=*ptr) != SquareNone; ptr++) { // HACK: no king
+      piece = board->square[from];
 
-         piece = board->square[from];
+      switch (PIECE_TYPE(piece)) {
 
-         switch (PIECE_TYPE(piece)) {
+      case Knight64:
 
-         case Knight64:
+         // mobility
 
-            // mobility
+         mob = -KnightUnit;
 
-            mob = -KnightUnit;
+         mob += unit[board->square[from-33]];
+         mob += unit[board->square[from-31]];
+         mob += unit[board->square[from-18]];
+         mob += unit[board->square[from-14]];
+         mob += unit[board->square[from+14]];
+         mob += unit[board->square[from+18]];
+         mob += unit[board->square[from+31]];
+         mob += unit[board->square[from+33]];
 
-            mob += unit[board->square[from-33]];
-            mob += unit[board->square[from-31]];
-            mob += unit[board->square[from-18]];
-            mob += unit[board->square[from-14]];
-            mob += unit[board->square[from+14]];
-            mob += unit[board->square[from+18]];
-            mob += unit[board->square[from+31]];
-            mob += unit[board->square[from+33]];
-
-            op[me] += mob * KnightMobOpening;
-            eg[me] += mob * KnightMobEndgame;
+         op[me] += mob * KnightMobOpening;
+         eg[me] += mob * KnightMobEndgame;
 
 			// mob = KnightPosMatrix[me][SquareTo64[from]]; 
 
 			// outpost
 			mob = 0;
-			if (me == White){
-				if (board->square[from-17] == WP)
-					mob += KnightOutpostMatrix[me][SquareTo64[from]]; 
-				if (board->square[from-15] == WP)
-					mob += KnightOutpostMatrix[me][SquareTo64[from]]; 
+			if (board->square[from+17] == BP)
+				mob += KnightOutpostMatrix[me][SquareTo64[from]]; 
+			if (board->square[from+15] == BP)
+				mob += KnightOutpostMatrix[me][SquareTo64[from]]; 
+			
+			penalty = 0;
+			if (((board->pawn_file[opp][FileA] | board->pawn_file[opp][FileB] | board->pawn_file[opp][FileC]) != 0)
+				&& ((board->pawn_file[opp][FileF] | board->pawn_file[opp][FileG] | board->pawn_file[opp][FileH]) != 0))
+				penalty -= EndgamePawnKnight;
+
+			if (((pawn_info->passed_bits[opp] & 224)) != 0 && ((pawn_info->passed_bits[opp] & 7) != 0))
+				penalty -= EndgamePassedPawnKnight;
+
+	
+			op[me] += mob;
+			//eg[me] += mob;
+			//op[me] += penalty / 2; 
+         eg[me] += penalty; 
+
+			if (PAWN_RANK(from,me) >= Rank5){
+				piece_nb++;
 			}
-			else{
-				if (board->square[from+17] == BP)
-					mob += KnightOutpostMatrix[me][SquareTo64[from]]; 
-				if (board->square[from+15] == BP)
-					mob += KnightOutpostMatrix[me][SquareTo64[from]]; 
-			} 
+
+         break;
+
+      case Bishop64:
+
+         // mobility
+
+         mob = -BishopUnit;
+			penalty = 0;
+
+         for (to = from-17; capture=board->square[to], THROUGH(capture); to -= 17) mob += MobMove;
+         mob += unit[capture];
+			if (capture == BP)
+				penalty += PawnBlockedBishop;
+
+         for (to = from-15; capture=board->square[to], THROUGH(capture); to -= 15) mob += MobMove;
+         mob += unit[capture];
+			if (capture == BP)
+				penalty += PawnBlockedBishop;
+
+         for (to = from+15; capture=board->square[to], THROUGH(capture); to += 15) mob += MobMove;
+         mob += unit[capture];
+			
+         for (to = from+17; capture=board->square[to], THROUGH(capture); to += 17) mob += MobMove;
+         mob += unit[capture];
+			
+         op[me] += mob * BishopMobOpening - penalty;
+         eg[me] += mob * BishopMobEndgame - penalty;
+
+			// outpost
+			mob = 0;
+			if (board->square[from+17] == BP)
+				mob += KnightOutpostMatrix[me][SquareTo64[from]]/2; 
+			if (board->square[from+15] == BP)
+				mob += KnightOutpostMatrix[me][SquareTo64[from]]/2; 
 
 			op[me] += mob;
-            // eg[me] += mob; 
+			//eg[me] += mob;
 
-            break;
+			if (PAWN_RANK(from,me) >= Rank5){
+				piece_nb++;
+			}
 
-         case Bishop64:
+         break;
 
-            // mobility
+      case Rook64:
 
-            mob = -BishopUnit;
+         // mobility
 
-            for (to = from-17; capture=board->square[to], THROUGH(capture); to -= 17) mob += MobMove;
-            mob += unit[capture];
+         mob = -RookUnit;
 
-            for (to = from-15; capture=board->square[to], THROUGH(capture); to -= 15) mob += MobMove;
-            mob += unit[capture];
+         for (to = from-16; capture=board->square[to], THROUGH(capture); to -= 16) mob += MobMove;
+         mob += unit[capture];
 
-            for (to = from+15; capture=board->square[to], THROUGH(capture); to += 15) mob += MobMove;
-            mob += unit[capture];
+         for (to = from- 1; capture=board->square[to], THROUGH(capture); to -=  1) mob += MobMove;
+         mob += unit[capture];
 
-            for (to = from+17; capture=board->square[to], THROUGH(capture); to += 17) mob += MobMove;
-            mob += unit[capture];
+         for (to = from+ 1; capture=board->square[to], THROUGH(capture); to +=  1) mob += MobMove;
+         mob += unit[capture];
 
-            op[me] += mob * BishopMobOpening;
-            eg[me] += mob * BishopMobEndgame;
+         for (to = from+16; capture=board->square[to], THROUGH(capture); to += 16) mob += MobMove;
+         mob += unit[capture];
 
-            break;
+         op[me] += mob * RookMobOpening;
+         eg[me] += mob * RookMobEndgame; 
 
-         case Rook64:
+         // open file
 
-            // mobility
+         if (UseOpenFile) {
 
-            mob = -RookUnit;
+            op[me] -= RookOpenFileOpening / 2;
+            eg[me] -= RookOpenFileEndgame / 2;
 
-            for (to = from-16; capture=board->square[to], THROUGH(capture); to -= 16) mob += MobMove;
-            mob += unit[capture];
+            rook_file = SQUARE_FILE(from);
 
-            for (to = from- 1; capture=board->square[to], THROUGH(capture); to -=  1) mob += MobMove;
-            mob += unit[capture];
+            if (board->pawn_file[me][rook_file] == 0) { // no friendly pawn
 
-            for (to = from+ 1; capture=board->square[to], THROUGH(capture); to +=  1) mob += MobMove;
-            mob += unit[capture];
+               op[me] += RookSemiOpenFileOpening;
+               eg[me] += RookSemiOpenFileEndgame;
 
-            for (to = from+16; capture=board->square[to], THROUGH(capture); to += 16) mob += MobMove;
-            mob += unit[capture];
+               if (board->pawn_file[opp][rook_file] == 0) { // no enemy pawn
+                  op[me] += RookOpenFileOpening - RookSemiOpenFileOpening;
+                  eg[me] += RookOpenFileEndgame - RookSemiOpenFileEndgame;
+               }
 
-            op[me] += mob * RookMobOpening;
-            eg[me] += mob * RookMobEndgame; 
+               if ((mat_info->cflags[opp] & MatKingFlag) != 0) {
 
-            // open file
+                  king = KING_POS(board,opp);
+                  king_file = SQUARE_FILE(king);
 
-            if (UseOpenFile) {
+                  delta = abs(rook_file-king_file); // file distance
 
-               op[me] -= RookOpenFileOpening / 2;
-               eg[me] -= RookOpenFileEndgame / 2;
-
-               rook_file = SQUARE_FILE(from);
-
-               if (board->pawn_file[me][rook_file] == 0) { // no friendly pawn
-
-                  op[me] += RookSemiOpenFileOpening;
-                  eg[me] += RookSemiOpenFileEndgame;
-
-                  if (board->pawn_file[opp][rook_file] == 0) { // no enemy pawn
-                     op[me] += RookOpenFileOpening - RookSemiOpenFileOpening;
-                     eg[me] += RookOpenFileEndgame - RookSemiOpenFileEndgame;
-                  }
-
-                  if ((mat_info->cflags[opp] & MatKingFlag) != 0) {
-
-                     king = KING_POS(board,opp);
-                     king_file = SQUARE_FILE(king);
-
-                     delta = abs(rook_file-king_file); // file distance
-
-                     if (delta <= 1) {
-                        op[me] += RookSemiKingFileOpening;
-                        if (delta == 0) op[me] += RookKingFileOpening - RookSemiKingFileOpening;
-                     }
+                  if (delta <= 1) {
+                     op[me] += RookSemiKingFileOpening;
+                     if (delta == 0) op[me] += RookKingFileOpening - RookSemiKingFileOpening;
                   }
                }
             }
+         }
 
-            // 7th rank
+         // 7th rank
 
 			if (PAWN_RANK(from,me) == Rank7) {
-               if ((pawn_info->flags[opp] & BackRankFlag) != 0 // opponent pawn on 7th rank
-                || PAWN_RANK(KING_POS(board,opp),me) == Rank8) {
-                  op[me] += Rook7thOpening;
-                  eg[me] += Rook7thEndgame;
-               }
-            } 
+					if ((pawn_info->flags[opp] & BackRankFlag) != 0 // opponent pawn on 7th rank
+					 || PAWN_RANK(KING_POS(board,opp),me) == Rank8) {
+						op[me] += Rook7thOpening;
+						eg[me] += Rook7thEndgame;
+					}
+			} 
 
-            break;
+			//if (PAWN_RANK(from,me) >= Rank7) op[me] += Rook7thOpening;
+			
+			if (PAWN_RANK(from,me) >= Rank5){
+				piece_nb++;
+			}
 
-         case Queen64:
+         break;
 
-            // mobility
+      case Queen64:
 
-            /* mob = -QueenUnit;
+         // mobility
 
-            for (to = from-17; capture=board->square[to], THROUGH(capture); to -= 17) mob += MobMove;
-            mob += unit[capture];
+/*         mob = -QueenUnit;
 
-            for (to = from-16; capture=board->square[to], THROUGH(capture); to -= 16) mob += MobMove;
-            mob += unit[capture];
+         for (to = from-17; capture=board->square[to], THROUGH(capture); to -= 17);// mob += MobMove;
+         mob += unit[capture];
 
-            for (to = from-15; capture=board->square[to], THROUGH(capture); to -= 15) mob += MobMove;
-            mob += unit[capture];
+         for (to = from-16; capture=board->square[to], THROUGH(capture); to -= 16);// mob += MobMove;
+         mob += unit[capture];
 
-            for (to = from- 1; capture=board->square[to], THROUGH(capture); to -=  1) mob += MobMove;
-            mob += unit[capture];
+         for (to = from-15; capture=board->square[to], THROUGH(capture); to -= 15);// mob += MobMove;
+         mob += unit[capture];
 
-            for (to = from+ 1; capture=board->square[to], THROUGH(capture); to +=  1) mob += MobMove;
-            mob += unit[capture];
+         for (to = from- 1; capture=board->square[to], THROUGH(capture); to -=  1);// mob += MobMove;
+         mob += unit[capture];
 
-            for (to = from+15; capture=board->square[to], THROUGH(capture); to += 15) mob += MobMove;
-            mob += unit[capture];
+         for (to = from+ 1; capture=board->square[to], THROUGH(capture); to +=  1);// mob += MobMove;
+         mob += unit[capture];
 
-            for (to = from+16; capture=board->square[to], THROUGH(capture); to += 16) mob += MobMove;
-            mob += unit[capture];
+         for (to = from+15; capture=board->square[to], THROUGH(capture); to += 15);// mob += MobMove;
+         mob += unit[capture];
 
-            for (to = from+17; capture=board->square[to], THROUGH(capture); to += 17) mob += MobMove;
-            mob += unit[capture]; */
+         for (to = from+16; capture=board->square[to], THROUGH(capture); to += 16);// mob += MobMove;
+         mob += unit[capture];
 
-            //op[me] += mob * QueenMobOpening;
-            //eg[me] += mob * QueenMobEndgame;
+         for (to = from+17; capture=board->square[to], THROUGH(capture); to += 17);// mob += MobMove;
+         mob += unit[capture]; 
+
+         op[me] += mob * QueenMobOpening;
+         eg[me] += mob * QueenMobEndgame;*/
 
 			king = KING_POS(board,opp);
 			king_file = SQUARE_FILE(king);
-            king_rank = SQUARE_RANK(king);
+			king_rank = SQUARE_RANK(king);
 			piece_file = SQUARE_FILE(from);
-            piece_rank = SQUARE_RANK(from); 
+			piece_rank = SQUARE_RANK(from); 
 						
-			new_mob = 10 - ((abs(king_file-piece_file) + abs(king_rank-piece_rank))); 
+			new_mob = 10 - (abs(king_file-piece_file) + abs(king_rank-piece_rank)); 
 			
-			op[me] += /* (mob * QueenMobOpening) + */ new_mob;
-            eg[me] += /* (mob * QueenMobEndgame) + */ new_mob; 
+			op[me] += new_mob;
+			eg[me] += new_mob; 
 
-            // 7th rank
+         // 7th rank
 
 			if (PAWN_RANK(from,me) == Rank7) {
-               if ((pawn_info->flags[opp] & BackRankFlag) != 0 // opponent pawn on 7th rank
-                || PAWN_RANK(KING_POS(board,opp),me) == Rank8) {
-                  op[me] += Queen7thOpening;
-                  eg[me] += Queen7thEndgame; 
-               }
-            } 
+					if ((pawn_info->flags[opp] & BackRankFlag) != 0 // opponent pawn on 7th rank
+					 || PAWN_RANK(KING_POS(board,opp),me) == Rank8) {
+						op[me] += Queen7thOpening;
+						eg[me] += Queen7thEndgame; 
+					}
+			}
+			
+			//if (PAWN_RANK(from,me) >= Rank7) op[me] += Queen7thOpening; 
+
+			if (PAWN_RANK(from,me) >= Rank5){
+				piece_nb++;
+			}
+
+			break;
 
-            break;
-         }
       }
    }
+  
+	op[me] += SSpaceWeight[piece_nb];
 
    // update
 
@@ -902,8 +1176,7 @@ static void eval_king(const board_t * board, const material_info_t * mat_info, i
    int piece;
    int attack_tot;
    int piece_nb;
-   int king_file, king_rank;
-
+   
    ASSERT(board!=NULL);
    ASSERT(mat_info!=NULL);
    ASSERT(opening!=NULL);
@@ -945,7 +1218,7 @@ static void eval_king(const board_t * board, const material_info_t * mat_info, i
 	  }
 	
 
-	  if (king_is_safe[White] == false){ 
+	  if (king_is_safe[White] == false && board->number[BlackQueen12] > 0){ 
 	  
 		  me = White;
 
@@ -955,11 +1228,11 @@ static void eval_king(const board_t * board, const material_info_t * mat_info, i
 		  
 		  // castling
 
-          penalty_2 = penalty_1;
+        penalty_2 = penalty_1;
 
-          if ((board->flags & FlagsWhiteKingCastle) != 0) {
-             tmp = shelter_square(board,mat_info,G1,me);
-             if (tmp < penalty_2) penalty_2 = tmp;
+        if ((board->flags & FlagsWhiteKingCastle) != 0) {
+				tmp = shelter_square(board,mat_info,G1,me);
+            if (tmp < penalty_2) penalty_2 = tmp;
 		  }
 
           if ((board->flags & FlagsWhiteQueenCastle) != 0) {
@@ -1005,7 +1278,7 @@ static void eval_king(const board_t * board, const material_info_t * mat_info, i
 		  } 
 	  }
 
-	  if (king_is_safe[Black] == false){ 
+	  if (king_is_safe[Black] == false && board->number[WhiteQueen12] > 0){ 
 
 	      me = Black;
 
@@ -1043,16 +1316,13 @@ static void eval_king(const board_t * board, const material_info_t * mat_info, i
    if (UseKingAttack) {
 
       for (colour = 0; colour < ColourNb; colour++) {
-
-         if ((mat_info->cflags[colour] & MatKingFlag) != 0) {
+			if ((mat_info->cflags[colour] & MatKingFlag) != 0) {
 
             me = colour;
             opp = COLOUR_OPP(me);
 
             king = KING_POS(board,me);
-			king_file = SQUARE_FILE(king);
-			king_rank = SQUARE_RANK(king);
-            
+			            
             // piece attacks
 
             attack_tot = 0;
@@ -1066,23 +1336,67 @@ static void eval_king(const board_t * board, const material_info_t * mat_info, i
                   piece_nb++;
                   attack_tot += KingAttackUnit[piece];
                }
-/*		       else{
-			       if ((abs(king_file-SQUARE_FILE(from)) + abs(king_rank-SQUARE_RANK(from))) <= 4){
-					   piece_nb++;
-                       attack_tot += KingAttackUnit[piece];
-				   }
-			   } */
             }
 
+				if (colour == 0){
+					if (board->square[king+16] == BP) {
+						piece_nb++;
+						attack_tot += KingAttackUnit[BP];
+					}
+					if (board->square[king+(16+1)] == BP) {
+						piece_nb++;
+						attack_tot += KingAttackUnit[BP];
+					}
+					if (board->square[king+(16-1)] == BP) {
+						piece_nb++;
+						attack_tot += KingAttackUnit[BP];
+					}
+					if (board->square[king+(16*2)] == BP) {
+						piece_nb++;
+						attack_tot += KingAttackUnit[BP];
+					}
+					if (board->square[king+(16*2+1)] == BP) {
+						piece_nb++;
+						attack_tot += KingAttackUnit[BP];
+					}
+					if (board->square[king+(16*2-1)] == BP) {
+						piece_nb++;
+						attack_tot += KingAttackUnit[BP];
+					}
+				}
+				else{
+					if (board->square[king-16] == WP) {
+						piece_nb++;
+						attack_tot += KingAttackUnit[WP];
+					}
+					if (board->square[king-(16+1)] == WP) {
+						piece_nb++;
+						attack_tot += KingAttackUnit[WP];
+					}
+					if (board->square[king-(16-1)] == WP) {
+						piece_nb++;
+						attack_tot += KingAttackUnit[WP];
+					}
+					if (board->square[king-(16*2)] == WP) {
+						piece_nb++;
+						attack_tot += KingAttackUnit[WP];
+					}
+					if (board->square[king-(16*2+1)] == WP) {
+						piece_nb++;
+						attack_tot += KingAttackUnit[WP];
+					}
+					if (board->square[king-(16*2-1)] == WP) {
+						piece_nb++;
+						attack_tot += KingAttackUnit[WP];
+					}
+				} 
+
             // scoring
 
             ASSERT(piece_nb>=0&&piece_nb<16);
 
-			if (op[colour] >= -22) //  Thomas king safe?
 				op[colour] -= (attack_tot * KingAttackOpening * KingAttackWeight[piece_nb]) / 256;
-			else
-				op[colour] -= (attack_tot * KingAttackOpening1 * KingAttackWeight[piece_nb]) / 256;
-			
+						
          }
       }
    }
@@ -1105,8 +1419,7 @@ static void eval_passer(const board_t * board, const pawn_info_t * pawn_info, in
    int sq;
    int min, max;
    int delta;
-   int white_passed_nb, black_passed_nb;
-   
+      
    ASSERT(board!=NULL);
    ASSERT(pawn_info!=NULL);
    ASSERT(opening!=NULL);
@@ -1119,9 +1432,6 @@ static void eval_passer(const board_t * board, const pawn_info_t * pawn_info, in
       eg[colour] = 0;
    }
 
-   white_passed_nb = 0;
-   black_passed_nb = 0;
-
    // passed pawns
 
    for (colour = 0; colour < ColourNb; colour++) {
@@ -1143,20 +1453,20 @@ static void eval_passer(const board_t * board, const pawn_info_t * pawn_info, in
          ASSERT(PIECE_IS_PAWN(board->square[sq]));
          ASSERT(COLOUR_IS(board->square[sq],att));
 
-		 // Thomas
+			// Thomas protected passed pawn bonus
 
-	/*	 if (att == White){
-             if (board->piece_size[Black]-1 == board->number[BlackKnight12] && (file == FileA || file == FileH)){
-				eg[att] += 30;
-			 }
-		
-		 }
-		 else{
-			 if (board->piece_size[White]-1 == board->number[WhiteKnight12] && (file == FileA || file == FileH)){
-				eg[att] += 30;
-			 }
-			 
-		 }  */
+		   if (att == White){
+	   		 if (board->square[sq-17] == WP || board->square[sq-15] == WP || board->square[sq-1] == WP || board->square[sq+1] == WP){
+	   			op[att] += ProtectedPassedOpening; 
+	   			eg[att] += ProtectedPassedEndgame;
+				 }
+			}
+   		else{
+	   		 if (board->square[sq+17] == BP || board->square[sq+15] == BP || board->square[sq-1] == BP || board->square[sq+1] == BP){
+	   			op[att] += ProtectedPassedOpening; 
+	   			eg[att] += ProtectedPassedEndgame;
+				 }
+			} 
 
          // opening scoring
 
@@ -1193,15 +1503,6 @@ static void eval_passer(const board_t * board, const pawn_info_t * pawn_info, in
 
    // update
 
-/*   if (white_passed_nb > 100){
-		op[White] = ((white_passed_nb / 10) * op[White]) / 100 + op[White];
-        eg[White] = ((white_passed_nb / 10) * eg[White]) / 100 + eg[White]; 
-   }
-   if (black_passed_nb > 100){
-		op[Black] = ((black_passed_nb / 10) * op[Black]) / 100 + op[Black];
-        eg[Black] = ((black_passed_nb / 10) * eg[Black]) / 100 + eg[Black]; 
-   } */
-   
    *opening += ((op[White] - op[Black]) * PassedPawnWeight) / 256;
    *endgame += ((eg[White] - eg[Black]) * PassedPawnWeight) / 256;
 }
@@ -1389,7 +1690,7 @@ static bool king_passer(const board_t * board, int pawn, int colour) {
 
 static bool free_passer(const board_t * board, int pawn, int colour) {
 
-   int me, opp;
+   int me;
    int inc;
    int sq;
    int move;
@@ -1399,8 +1700,7 @@ static bool free_passer(const board_t * board, int pawn, int colour) {
    ASSERT(COLOUR_IS_OK(colour));
 
    me = colour;
-   opp = COLOUR_OPP(me);
-
+   
    inc = PAWN_MOVE_INC(me);
    sq = pawn + inc;
    ASSERT(SQUARE_IS_OK(sq));
@@ -2020,8 +2320,8 @@ static int shelter_square(const board_t * board, const material_info_t * mat_inf
 
     /* Thomas piece exchange if king is not safe */
    
-   if (KingSafety && penalty > 22 && mat_info->pv[COLOUR_OPP(colour)] > KingSafetyMargin)
-	   penalty = (penalty * (mat_info->pv[COLOUR_OPP(colour)]/(KingSafetyMargin/100))) / 100; 
+/*   if (KingSafety && penalty > 22 && mat_info->pv[COLOUR_OPP(colour)] > KingSafetyMargin)
+	   penalty = (penalty * (mat_info->pv[COLOUR_OPP(colour)]/(KingSafetyMargin/100))) / 100; */
 
    if (penalty == 0) penalty = 11; // weak back rank
 
