@@ -18,7 +18,7 @@
 // constants
 
 static const bool UseTable = true;
-static const uint32 TableSize = 256; // 4kB
+static const uint32 TableSize = 16384; // was 256 4kB
 
 static const int PawnPhase   = 0;
 static const int KnightPhase = 1;
@@ -38,9 +38,9 @@ static /* const */ int EndgameExchangePenalty = 30;
 static /* const */ int MaterialWeight = 256; // 100%
 
 static const int PawnOpening   = 70; // was 100 80
-static const int PawnEndgame   = 90; // was 100
+static const int PawnEndgame   = 90; // was 100 90
 static const int KnightOpening = 325;
-static const int KnightEndgame = 325;
+static const int KnightEndgame = 315; // was 325
 static const int BishopOpening = 325;
 static const int BishopEndgame = 325;
 static const int RookOpening   = 500;
@@ -202,7 +202,7 @@ static void material_comp_info(material_info_t * info, const board_t * board) {
    int phase;
    int opening, endgame;
    int owf,obf,ewf,ebf; /* Thomas */
-   int WhiteMinors,BlackMinors,WhiteMajors,BlackMajors
+   int WhiteMinors,BlackMinors,WhiteMajors,BlackMajors;
 
    ASSERT(info!=NULL);
    ASSERT(board!=NULL);
@@ -676,7 +676,7 @@ static void material_comp_info(material_info_t * info, const board_t * board) {
    else if (obf > owf && wp > bp){
 	   opening -= OpeningExchangePenalty;
 	   endgame -= OpeningExchangePenalty; 
-   }  */
+   } */
 /*   if (WhiteMinors != BlackMinors) {
 		if (WhiteMajors == BlackMajors) {
 		  if (WhiteMinors > BlackMinors){
