@@ -3,6 +3,7 @@
 
 // includes
 
+#include "pawn.h"
 #include "attack.h"
 #include "board.h"
 #include "colour.h"
@@ -23,12 +24,12 @@
 #include "trans.h"
 #include "util.h"
 #include "value.h"
+#include "probe.h"
 
 #define ABS(x) ((x)<0?-(x):(x))
 #define MIN(X, Y)  ((X) < (Y) ? (X) : (Y))
 #define R_adpt(max_pieces, depth, Reduction) (Reduction + ((depth) > (9 + ((max_pieces<3) ? 2 : 0))))
 
-
 // constants and variables
 
 // main search
@@ -66,11 +67,12 @@ static const bool ExtendSingleReply = true; // true
 // history pruning
 
 static /* const */ bool UseHistory = true;
+static /* const */ bool UseHistoryPruning = true;
 static const int HistoryDepth = 3; // was 3
 static const int HistoryMoveNb = 3; // was 3
 static /* const */ int HistoryValue = 9830; // 60%
 static /* const */ int HistoryBound = 2458; // * 16384 + 50) / 100 10%=1638 15%=2458 20%=3277
-static /* const */ bool UseExtendedHistory = true;
+static /* const */ bool UseExtendedHistory = false;
 static const bool HistoryReSearch = true;
 
 // futility pruning
@@ -78,8 +80,8 @@ static const bool HistoryReSearch = true;
 static /* const */ bool UseFutility = true; // false
 static const int FutilityMargin = 100;
 static /* const */ int FutilityMargin1 = 100;
-static /* const */ int FutilityMargin2 = 300;
-static /* const */ int FutilityMargin3 = 950;
+static /* const */ int FutilityMargin2 = 200;
+static /* const */ int FutilityMargin3 = 300;
 
 // quiescence search
 
@@ -107,7 +109,7 @@ static int  full_root            (list_t * list, board_t * board, int alpha, int
 static int  full_search          (board_t * board, int alpha, int beta, int depth, int height, mv_t pv[], int node_type);
 static int  full_no_null         (board_t * board, int alpha, int beta, int depth, int height, mv_t pv[], int node_type, int trans_move, int * best_move);
 
-static int  full_quiescence      (board_t * board, int alpha, int beta, int depth, int height, mv_t pv[]);
+static int  full_quiescence      (board_t * board, int alpha, int beta, int depth, int height, mv_t pv[], bool in_pv);
 
 static int  full_new_depth       (int depth, int move, board_t * board, bool single_reply, bool mate_threat, bool in_pv, int height);
 
@@ -123,6 +125,8 @@ static bool simple_stalemate     (const board_t * board);
 
 static bool passed_pawn_move     (int move, const board_t * board);
 
+static bool is_passed				(const board_t * board, int to);
+
 // functions
 
 // search_full_init()
@@ -233,18 +237,18 @@ int search_full_root(list_t * list, board_t * board, int depth, int search_type)
    ASSERT(board_is_legal(board));
    ASSERT(depth>=1);
 
-   if (SearchBest[SearchCurrent->multipv].value == 0){
+/*   if (SearchBest[SearchCurrent->multipv].value == 0){
 	   a = -ValueInf;
 	   b = +ValueInf;
    }
-   else{
+   else{ */
 	   a = SearchBest[SearchCurrent->multipv].value - 40;
 	   b = SearchBest[SearchCurrent->multipv].value + 40;
-   }
+ //  }
 
-   if (SearchInput->multipv > 0){
+   if (search_type == SearchShort || SearchInput->multipv > 0){
 	   a = -ValueInf;
-       b = +ValueInf;
+      b = +ValueInf;
    }
 
    value = full_root(list,board,a,b,depth,0,search_type);
@@ -286,6 +290,8 @@ static int full_root(list_t * list, board_t * board, int alpha, int beta, int de
    SearchStack[height].move = MoveNone;
    SearchStack[height].threat_move = MoveNone;
    SearchStack[height].reduced = false;
+	SearchStack[height].extended = false;
+	SearchStack[height].capture = false;
 
    SearchCurrent->node_nb++;
    SearchInfo->check_nb--;
@@ -296,6 +302,10 @@ static int full_root(list_t * list, board_t * board, int alpha, int beta, int de
    old_alpha = alpha;
    best_value[SearchCurrent->multipv] = ValueNone;
 
+/*	if (depth == 1){
+		SearchStack[height].eval_value = full_quiescence(board,alpha,beta,0,height,new_pv,node_type==NodePV);
+	} */
+
    // move loop
 
    for (i = 0; i < LIST_SIZE(list); i++) {
@@ -323,16 +333,28 @@ static int full_root(list_t * list, board_t * board, int alpha, int beta, int de
       search_update_root();
 
       new_depth = full_new_depth(depth,move,board,board_is_check(board)&&LIST_SIZE(list)==1,false,true, height);
+		if (new_depth == depth)
+			SearchStack[height].extended = true;
+		else
+			SearchStack[height].extended = false;
 
       move_do(board,move,undo);
 
       if (search_type == SearchShort || best_value[SearchCurrent->multipv] == ValueNone) { // first move
-         value = -full_search(board,-beta,-alpha,new_depth,height+1,new_pv,NodePV);
-		 if (value <= alpha){ // research
-			 old_alpha = -ValueInf;
-			 value = -full_search(board,-beta,ValueInf,new_depth,height+1,new_pv,NodePV);
-		 }
+			value = -full_search(board,-beta,-alpha,new_depth,height+1,new_pv,NodePV);
+			if (value <= alpha){ // research
+				old_alpha = alpha = -ValueInf;
+				SearchRoot->change = true;
+				SearchRoot->easy = false;
+				SearchRoot->flag = false;
+				search_update_root();
+				value = -full_search(board,-beta,ValueInf,new_depth,height+1,new_pv,NodePV);
+			}
 		 else if (value >= beta){ // research
+			 SearchRoot->change = true;
+          SearchRoot->easy = false;
+          SearchRoot->flag = false;
+          search_update_root();
 			 value = -full_search(board,-ValueInf,-alpha,new_depth,height+1,new_pv,NodePV);
 		 }
 			  
@@ -343,10 +365,12 @@ static int full_root(list_t * list, board_t * board, int alpha, int beta, int de
             SearchRoot->easy = false;
             SearchRoot->flag = false;
             search_update_root();
-            value = -full_search(board,-beta,-alpha,new_depth,height+1,new_pv,NodePV);
+            value = -full_search(board,-beta-50,-alpha,new_depth,height+1,new_pv,NodePV);
+				if (value >= beta) // research
+					value = -full_search(board,-ValueInf,-alpha,new_depth,height+1,new_pv,NodePV);
          }
       }
-
+		
       move_undo(board,move,undo);
 
       if (value <= alpha) { // upper bound
@@ -360,7 +384,7 @@ static int full_root(list_t * list, board_t * board, int alpha, int beta, int de
       if (value > best_value[SearchCurrent->multipv] && (best_value[SearchCurrent->multipv] == ValueNone || value > alpha)) {
 
          SearchBest[SearchCurrent->multipv].move = move;
-		 SearchStack[height].best_move = move;
+		   SearchStack[height].best_move = move;
          SearchBest[SearchCurrent->multipv].value = value;
          if (value <= alpha) { // upper bound
             SearchBest[SearchCurrent->multipv].flags = SearchUpper;
@@ -414,8 +438,8 @@ static int full_search(board_t * board, int alpha, int beta, int depth, int heig
    int played_nb;
    int i;
    int opt_value;
+	int probe_score, probe_depth;
    bool reduced;
-   int mb;
    attack_t attack[1];
    sort_t sort[1];
    undo_t undo[1];
@@ -434,13 +458,24 @@ static int full_search(board_t * board, int alpha, int beta, int depth, int heig
 
    // horizon?
 
-   if (depth <= 0){
+   if (depth <= 0 || node_type == NodePV){
 	   if (node_type == NodePV)
 			CheckDepth = 1 - CheckNb - 1;
 	   else
 			CheckDepth = 1 - CheckNb;
-	   return full_quiescence(board,alpha,beta,0,height,pv);
-   }
+		if (depth <= 0){
+			SearchStack[height].eval_value = full_quiescence(board,alpha,beta,0,height,pv,node_type==NodePV);
+			return SearchStack[height].eval_value; 
+		}
+/*		else{
+			SearchStack[height].eval_value = full_quiescence(board,alpha,beta,0,height,new_pv, node_type==NodePV);
+			if(height >= 2){
+				if (SearchStack[height].eval_value > SearchStack[height-2].eval_value+25 && SearchStack[height-1].extended == false){
+					depth++;
+				}
+			}
+		} */
+	}
 
    // init
 
@@ -448,6 +483,8 @@ static int full_search(board_t * board, int alpha, int beta, int depth, int heig
    SearchStack[height].move = MoveNone;
    SearchStack[height].threat_move = MoveNone;
    SearchStack[height].reduced = false;
+	SearchStack[height].extended = false;
+	SearchStack[height].capture = false;
    mate_threat = false;
 
    SearchCurrent->node_nb++;
@@ -463,7 +500,32 @@ static int full_search(board_t * board, int alpha, int beta, int depth, int heig
 
    // draw?
 
-   if (board_is_repetition(board) || recog_draw(board)) return ValueDraw;
+	if (board_is_repetition(board)) return ValueDraw;
+
+		/*
+		Interior node recognizer from scorpio by Daniel Shawul
+		-> dont probe at the leaves as this will slow down search
+	    For 4/3 pieces probe there also.
+		-> After captures and pawn moves assume exact score and cutoff tree,
+	    because we are making progress. Note this is not done only for speed.
+		-> if we are far from root (depth / 2), assume exact score and cutoff tree
+	*/
+
+	if (egbb_is_loaded && board->piece_nb <=5){ 
+		probe_depth = (2 * SearchCurrent->max_extensions) / 3;
+		if ((board->piece_nb <=4 || height <= probe_depth)
+           && (height >= probe_depth 
+		         || board->cap_sq != MoveNone || PIECE_IS_PAWN(board->square[MOVE_TO(SearchStack[height-1].move)]))
+			  ) {
+		
+			if (probe_bitbases(board, probe_score)){
+				probe_score = value_from_trans(probe_score,height);
+				return probe_score;
+			}
+		}
+	}
+
+	if (recog_draw(board)) return ValueDraw;
 
    // mate-distance pruning
 
@@ -533,7 +595,7 @@ static int full_search(board_t * board, int alpha, int beta, int depth, int heig
 
    // height limit
 
-   if (height >= HeightMax-1) return eval(board, alpha, beta);
+   if (height >= HeightMax-1) return eval(board, alpha, beta, node_type==NodePV);
 
    // more init
 
@@ -547,19 +609,21 @@ static int full_search(board_t * board, int alpha, int beta, int depth, int heig
 
    // null-move pruning
 
-   if (UseNull && depth >= NullDepth && node_type != NodePV) {
+   if (UseNull && depth >= NullDepth && (node_type != NodePV || height%2 != 0)) {
 
       if (!in_check
        && !value_is_mate(beta)
        && do_null(board)
-       && (!UseNullEval || depth <= NullReduction+1 || eval(board,alpha, beta) >= beta)) {
+       && (!UseNullEval || depth <= NullReduction+1 || eval(board,alpha, beta, node_type==NodePV) >= beta)) {
 
          // null-move search
+			new_depth = depth - NullReduction - 1;
+	   	 //new_depth = depth - R_adpt(board->piece_size[board->turn]+board->pawn_size[board->turn],depth,NullReduction) - 1;
 		 
-         new_depth = depth - NullReduction - 1;
-		 //new_depth = depth - R_adpt(board->piece_size[board->turn]+board->pawn_size[board->turn],depth,NullReduction) - 1;
-		 
-	     move_do_null(board,undo);
+	      move_do_null(board,undo);
+
+			SearchStack[height].move = SearchStack[height-1].move;
+
          value = -full_search(board,-beta,-beta+1,new_depth,height+1,new_pv,NODE_OPP(node_type));
          move_undo_null(board,undo);
 
@@ -575,21 +639,24 @@ static int full_search(board_t * board, int alpha, int beta, int depth, int heig
                value = full_no_null(board,alpha,beta,new_depth,height,new_pv,NodeCut,trans_move,&move);
 
                if (value >= beta) {
-                  ASSERT(move==new_pv[0]);
-                  played[played_nb++] = move;
-                  best_move = move;
-				  SearchStack[height].move = move;
-				  SearchStack[height].best_move = move;
-                  best_value = value;
-                  pv_copy(pv,new_pv);
-                  goto cut;
+					   ASSERT(move==new_pv[0]);
+						played[played_nb++] = move;
+						best_move = move;
+						SearchStack[height].best_move = move;
+						best_value = value;
+						pv_copy(pv,new_pv);
+						goto cut;
                }
+/*					else{ 
+						if (!SearchStack[height-1].extended && node_type == NodePV)
+							depth++;
+					} */
             }
          }
 
          // pruning
 
-         if (value >= beta) {
+         if (value >= beta && node_type != NodePV) {
 
             if (value > +ValueEvalInf) value = +ValueEvalInf; // do not return unproven mates
             ASSERT(!value_is_mate(value));
@@ -600,7 +667,8 @@ static int full_search(board_t * board, int alpha, int beta, int depth, int heig
             best_value = value;
             goto cut;
          }
-		 SearchStack[height].threat_move = SearchStack[height+1].best_move;
+			
+			SearchStack[height].threat_move = SearchStack[height+1].best_move;
 	/*	 if (SearchStack[height-1].reduced){ // Idea by Tord Romstad 
 				if (MOVE_FROM(SearchStack[height+1].best_move) == MOVE_TO(SearchStack[height-1].move))
 					return alpha-1; */
@@ -615,9 +683,9 @@ static int full_search(board_t * board, int alpha, int beta, int depth, int heig
    }
 
    // mate threat 
-/*   mate_threat = false;
-   if (value <= VALUE_MATE(height+2)){ 
-			mate_threat = true;
+/*   if (value <= VALUE_MATE(HeightMax-1)){ 
+			if (!SearchStack[height-1].extended && node_type == NodePV)
+							depth++;
    } */
 
    // Internal Iterative Deepening
@@ -645,19 +713,45 @@ static int full_search(board_t * board, int alpha, int beta, int depth, int heig
 
    opt_value = +ValueInf;
 
-   while ((move=sort_next(sort)) != MoveNone) {
+   while ((move=sort_next(sort,height)) != MoveNone) {
 
-	  SearchStack[height].move = move;
+		SearchStack[height].move = move;
 
-	  // history_tried(move,board);
-	  
-      // extensions
+	   // extensions
 
       new_depth = full_new_depth(depth,move,board,single_reply,mate_threat,node_type==NodePV, height);
+		if (new_depth == depth)
+			SearchStack[height].extended = true;
+		else
+			SearchStack[height].extended = false;
 	  
-      // futility pruning
+      // history pruning
 
-	  if (UseFutility && depth <= 2 && node_type != NodePV) {
+		reduced = false;
+
+		value = sort->value; // history score
+
+/*		if (value > ((90 * 16384 + 50) / 100) && value < 16384 && new_depth < depth && SearchStack[height-1].extended == false && node_type == NodePV){
+			new_depth++;
+			SearchStack[height].extended = true;
+		}*/
+
+/*		if (new_depth < depth && SearchStack[height-1].extended == false && node_type == NodePV && played_nb < HistoryMoveNb){
+			new_depth++;
+			SearchStack[height].extended = true;
+		}*/
+
+      if (UseHistoryPruning && !in_check /* && depth < HistoryDepth */ && depth <= SearchCurrent->max_extensions/2 /*depth <= 4*/ 
+			&& (node_type != NodePV /*|| UseExtendedHistory*/) 
+			&& played_nb >= HistoryMoveNb /* && height%2 == 0 */
+			&& new_depth < depth && value < (depth <= 2 ? HistoryValue / 2 : HistoryValue / 3)
+			&& !move_is_dangerous(move,board) && !move_is_tactical(move,board)){ 
+				continue; 
+		}
+		
+		// futility pruning
+
+	  if (UseFutility && depth <= 3 && node_type != NodePV) {
 		  
          if (!in_check && new_depth < depth && !move_is_tactical(move,board) && !move_is_dangerous(move,board)) {
 
@@ -666,14 +760,15 @@ static int full_search(board_t * board, int alpha, int beta, int depth, int heig
             // optimistic evaluation
 
             if (opt_value == +ValueInf) {
-				if (depth==2){
-					FutilityMargin = FutilityMargin2;
-				}
-				else{
-					FutilityMargin = FutilityMargin1;
-				}
-				opt_value = eval(board,alpha,beta) + FutilityMargin;
-				ASSERT(opt_value<+ValueInf);
+					if (depth == 1)
+						FutilityMargin = FutilityMargin1;
+					else if (depth == 2)
+						FutilityMargin = FutilityMargin2;
+					else
+						FutilityMargin = FutilityMargin3;
+					
+					opt_value = eval(board,alpha,beta,node_type==NodePV) + FutilityMargin;
+					ASSERT(opt_value<+ValueInf);
             }
 
             value = opt_value;
@@ -692,52 +787,21 @@ static int full_search(board_t * board, int alpha, int beta, int depth, int heig
          }
       } 
 
-	  // history pruning
-
-/*      reduced = false;
-
-      if (UseHistory && depth >= HistoryDepth && node_type != NodePV) {
-         if (!in_check && played_nb >= HistoryMoveNb && new_depth < depth) {
-            ASSERT(best_value!=ValueNone);
-            ASSERT(played_nb>0);
-            ASSERT(sort->pos>0&&move==LIST_MOVE(sort->list,sort->pos-1));
-            if (history_reduction(move,board) == true) {
-               ASSERT(value>=0&&value<16384);
-               ASSERT(move!=trans_move);
-               ASSERT(!move_is_tactical(move,board));
-               ASSERT(!move_is_check(move,board));
-               new_depth--;
-               reduced = true;
-            }
-         }
-      } */
-
-	  // history pruning
-
-      reduced = false;
-	  value = sort->value; // history score
-      if (!in_check && depth <= 4 && node_type != NodePV && new_depth < depth && value < (depth <= 3 ? HistoryValue / 3 : HistoryValue / 4)) 
-		  continue;
-
-
-      if (UseHistory && depth >= HistoryDepth && node_type != NodePV) {
+      if (UseHistory && depth >= HistoryDepth) {
          if (!in_check && played_nb >= HistoryMoveNb && new_depth < depth) {
             ASSERT(best_value!=ValueNone);
             ASSERT(played_nb>0);
             ASSERT(sort->pos>0&&move==LIST_MOVE(sort->list,sort->pos-1));
             value = sort->value; // history score
-            if (value < HistoryValue) {
+            if (value < HistoryValue && (node_type != NodePV || (UseExtendedHistory && value < HistoryValue / 2)) &&
+					 !move_is_dangerous(move,board) /* && !move_is_tactical(move,board)*/) {
                ASSERT(value>=0&&value<16384);
                ASSERT(move!=trans_move);
                ASSERT(!move_is_tactical(move,board));
                ASSERT(!move_is_check(move,board));
                new_depth--;
                reduced = true;
-			   if (UseExtendedHistory && value < HistoryValue / 2 && depth >= 8){
-				   new_depth--;
-				   
 			   }
-            }
          }
       }   
 
@@ -820,10 +884,10 @@ cut:
          for (i = 0; i < played_nb-1; i++) {
             move = played[i];
             ASSERT(move!=best_move);
-            history_bad(move,board);
+            history_bad(move,board,height);
          }
 
-         history_good(best_move,board);
+         history_good(best_move,board,height);
       }
    }
 
@@ -869,11 +933,6 @@ static int full_no_null(board_t * board, int alpha, int beta, int depth, int hei
 
    // init
 
-   SearchStack[height].best_move = MoveNone;
-   SearchStack[height].move = MoveNone;
-   SearchStack[height].threat_move = MoveNone;
-   SearchStack[height].reduced = false;
-
    SearchCurrent->node_nb++;
    SearchInfo->check_nb--;
    PV_CLEAR(pv);
@@ -895,11 +954,15 @@ static int full_no_null(board_t * board, int alpha, int beta, int depth, int hei
 
    sort_init(sort,board,attack,depth,height,trans_move);
 
-   while ((move=sort_next(sort)) != MoveNone) {
+   while ((move=sort_next(sort,height)) != MoveNone) {
 
 	  SearchStack[height].move = move;
 
       new_depth = full_new_depth(depth,move,board,false,false,false,height);
+		if (new_depth == depth)
+			SearchStack[height].extended = true;
+		else
+			SearchStack[height].extended = false;
 
       move_do(board,move,undo);
       value = -full_search(board,-beta,-alpha,new_depth,height+1,new_pv,NODE_OPP(node_type));
@@ -933,20 +996,19 @@ cut:
 
 // full_quiescence()
 
-static int full_quiescence(board_t * board, int alpha, int beta, int depth, int height, mv_t pv[]) {
+static int full_quiescence(board_t * board, int alpha, int beta, int depth, int height, mv_t pv[], bool in_pv) {
 
    bool in_check;
-   int old_alpha;
+   int old_alpha, probe_score;
    int value, best_value;
    int best_move;
    int move;
-   int opt_value;
+   int opt_value, probe_depth;
    attack_t attack[1];
    sort_t sort[1];
    undo_t undo[1];
    mv_t new_pv[HeightMax];
-   int cd;
-
+   
    ASSERT(board!=NULL);
    ASSERT(range_is_ok(alpha,beta));
    ASSERT(depth_is_ok(depth));
@@ -962,6 +1024,8 @@ static int full_quiescence(board_t * board, int alpha, int beta, int depth, int
    SearchStack[height].move = MoveNone;
    SearchStack[height].threat_move = MoveNone;
    SearchStack[height].reduced = false;
+	SearchStack[height].extended = false;
+	SearchStack[height].capture = false;
 
    SearchCurrent->node_nb++;
    SearchInfo->check_nb--;
@@ -976,7 +1040,32 @@ static int full_quiescence(board_t * board, int alpha, int beta, int depth, int
 
    // draw?
 
-   if (board_is_repetition(board) || recog_draw(board)) return ValueDraw;
+   if (board_is_repetition(board)) return ValueDraw;
+
+	/*
+		Interior node recognizer from scorpio by Daniel Shawul
+		-> dont probe at the leaves as this will slow down search
+	    For 4/3 pieces probe there also.
+		-> After captures and pawn moves assume exact score and cutoff tree,
+	    because we are making progress. Note this is not done only for speed.
+		-> if we are far from root (depth / 2), assume exact score and cutoff tree
+	*/
+
+	if (egbb_is_loaded && board->piece_nb <=5){ 
+		probe_depth = (2 * SearchCurrent->max_extensions) / 3;
+		if ((board->piece_nb <=4 || height <= probe_depth)
+           && (height >= probe_depth 
+		         || board->cap_sq != MoveNone || PIECE_IS_PAWN(board->square[MOVE_TO(SearchStack[height-1].move)]))
+			  ) {
+		
+			if (probe_bitbases(board, probe_score)){
+				probe_score = value_from_trans(probe_score,height);
+				return probe_score;
+			}
+		}
+	}
+
+	if (recog_draw(board)) return ValueDraw;
 
    // mate-distance pruning
 
@@ -1014,7 +1103,7 @@ static int full_quiescence(board_t * board, int alpha, int beta, int depth, int
 
    // height limit
 
-   if (height >= HeightMax-1) return eval(board, alpha, beta);
+   if (height >= HeightMax-1) return eval(board, alpha, beta, in_pv);
 
    // more init
 
@@ -1032,7 +1121,7 @@ static int full_quiescence(board_t * board, int alpha, int beta, int depth, int
 
       // stand pat
 
-      value = eval(board, alpha, beta);
+      value = eval(board, alpha, beta, in_pv);
 
       ASSERT(value>best_value);
       best_value = value;
@@ -1097,7 +1186,7 @@ static int full_quiescence(board_t * board, int alpha, int beta, int depth, int
       }
 
       move_do(board,move,undo);
-      value = -full_quiescence(board,-beta,-alpha,depth-1,height+1,new_pv);
+      value = -full_quiescence(board,-beta,-alpha,depth-1,height+1,new_pv,in_pv);
       move_undo(board,move,undo);
 
       if (value > best_value) {
@@ -1142,40 +1231,57 @@ static int full_new_depth(int depth, int move, board_t * board, bool single_repl
 
    new_depth = depth - 1;
    
+	SearchStack[height].capture = false;
+	
    if (in_pv && board->square[MOVE_TO(move)] != Empty && !PIECE_IS_PAWN(board->square[MOVE_TO(move)])){
 	   if ((board->piece_size[White] + board->piece_size[Black]) == 3){
-			return new_depth+3;
+			return new_depth+1;
 	   }
 	   else if ((board->piece_size[White] == 3 && board->piece_size[Black] == 2) 
 		   || (board->piece_size[White] == 2 && board->piece_size[Black] == 3))
 		   return new_depth+1; 
    }
-   //else{ 
-		if ((single_reply && ExtendSingleReply)
-			|| (in_pv && MOVE_TO(move) == board->cap_sq // recapture
-		/*		&& (see_move(move,board) > 0 || ABS(VALUE_PIECE(board->square[MOVE_TO(move)])-VALUE_PIECE(board->square[MOVE_FROM(move)])) <= 250 ) */)  
+   if ((single_reply && ExtendSingleReply)
+			|| (/*in_pv &&*/ MOVE_TO(move) == board->cap_sq // recapture
+				 && (see_move(move,board) > 0 /*|| VALUE_PIECE(board->square[MOVE_TO(move)])==VALUE_PIECE(board->square[MOVE_FROM(move)])*/)) 
+		//	|| (in_pv && MOVE_TO(move) == board->cap_sq) // recapture
 			|| (in_pv && PIECE_IS_PAWN(MOVE_PIECE(move,board))
-				  && (PAWN_RANK(MOVE_TO(move),board->turn) == Rank7 || PAWN_RANK(MOVE_TO(move),board->turn) == Rank6)
-				  /* && see_move(move,board) >= 0 */)
-			|| (in_pv &&  board->square[MOVE_TO(move)] != PieceNone256 && SearchCurrent->max_extensions-height >= 6) 
-			//|| (in_pv &&  depth <= 1 && height <= SearchCurrent->max_extensions)
-			//|| (in_pv && mate_threat == true)
+				  && (PAWN_RANK(MOVE_TO(move),board->turn) == Rank7 || PAWN_RANK(MOVE_TO(move),board->turn) == Rank6))
+		//		  /* && see_move(move,board) >= 0 */)
+		//	|| (in_pv &&  board->square[MOVE_TO(move)] != PieceNone256 && 
+		//	    SearchCurrent->max_extensions-height >= 6) 
+			//|| (in_pv &&  board->square[MOVE_TO(move)] != PieceNone256 && SearchCurrent->max_extensions) 
 			|| move_is_check(move,board)) {
 			return new_depth+1;
-		}
-	//	else{
-			if (in_pv && PIECE_IS_PAWN(MOVE_PIECE(move,board))){
-				if (passed_pawn_move(move,board))
-					return new_depth+1;
-			}
-	//	}
-//   }
-  
-   ASSERT(new_depth>=0&&new_depth<=depth);
+	}
+
+if (in_pv && PIECE_IS_PAWN(MOVE_PIECE(move,board))){
+		if (passed_pawn_move(move,board))
+			return new_depth+1;
+	}
+
+/*	if (in_pv && board->square[MOVE_TO(move)] != PieceNone256 && see_move(move,board) > 0){
+			return new_depth+1;
+	} */
+
+//	if (in_pv &&  board->square[MOVE_TO(move)] != PieceNone256 && height > 0 && (SearchStack[height-1].capture == false /* || height%2 != 0 */)){
+//		SearchStack[height].capture = true;
+//		return new_depth+1;
+//	} 
+
+/*	if (in_pv && mate_threat){
+		if (height == 0)
+			return new_depth+1;
+		else if (SearchStack[height-1].extended == false)
+			return new_depth+1;
+	} */
+
+	ASSERT(new_depth>=0&&new_depth<=depth);
 
    return new_depth;
 }
 
+
 // do_null()
 
 static bool do_null(const board_t * board) {
@@ -1246,6 +1352,9 @@ static bool move_is_dangerous(int move, const board_t * board) {
       return true;
    }
 
+//	if (is_passed(board,MOVE_TO(move)) && PAWN_RANK(MOVE_TO(move),board->turn) >= Rank6)
+//		return true;
+
    return false;
 }
 
@@ -1352,6 +1461,31 @@ static bool passed_pawn_move(int move, const board_t * board) {
    return true;
 }
 
+static bool is_passed(const board_t * board, int to) { 
+
+   int t2; 
+   int me, opp;
+   int file, rank;
+
+   me = board->turn; 
+   opp = COLOUR_OPP(me);
+   file = SQUARE_FILE(to);
+   rank = PAWN_RANK(to,me);
+ 
+   t2 = board->pawn_file[me][file] | BitRev[board->pawn_file[opp][file]]; 
+
+   // passed pawns 
+   if ((t2 & BitGT[rank]) == 0) { 
+ 
+    if (((BitRev[board->pawn_file[opp][file-1]] | BitRev[board->pawn_file[opp][file+1]]) & BitGT[rank]) == 0) { 
+        return true;
+
+       } 
+   } 
+
+   return false;
+ 
+}
 
 // end of search_full.cpp
 
